/**************************************************************************
*                                                                         *
*              National Center for Biotechnology Information              *
*       Bldg. 38A, NIH,  8600 Rockville Pike,  Bethesda, MD  20894        *
*                                                                         *
*                             COPYRIGHT NOTICE                            *
*                                                                         *
* This software/database is categorized as "United States Government      *
* Work" under the terms of the United States Copyright Act.  It was       *
* produced as part of the author's official duties as a Government        *
* employee and thus can not be copyrighted.  This software/database is    *
* freely available to the public for use without a copyright notice.      *
* Restrictions can not be placed on its present or future use.            *
*                                                                         *
* Although all reasonable efforts have been taken to ensure the accuracy  *
* and reliability of the software and data, the National Library of       *
* Medicine (NLM) and the U.S. Government do not and can not warrant the   *
* performance or results that may be obtained by using this software,     *
* data, or derivative works thereof.  The NLM and the U.S. Government     *
* disclaim any and all warranties, expressed or implied, as to the        *
* performance, merchantability or fitness for any particular purpose or   *
* use.                                                                    *
*                                                                         *
* In any work or product derived from this material, proper attribution   *
* of the author(s) as the source of the software or data would be         *
* appreciated.                                                            *
*                                                                         *

Karlin, Samuel and Stephen F. Altschul (1990).  Methods for assessing
the statistical significance of molecular sequence features by using
general scoring schemes.  Proc. Natl. Acad. Sci. USA 87:2264-2268.

Altschul, Stephen F., Warren Gish, Webb Miller, Eugene W. Myers, and
David J. Lipman (1990).  A basic local alignment search tool.
J. Mol. Biol. 215:403-410.
**************************************************************************/
/* (For a better look, set tabstops every 4 columns) */
#ifndef __BLAST_H__
#define __BLAST_H__
#include <signal.h>
#include <gish.h>

/* EXTERN should be defined in the main program module only */
#ifndef EXTERN
#define EXTERN	extern
#ifndef INITIAL
#define INITIAL(x)
#endif
#else
#ifndef INITIAL
#define INITIAL(x) = x
#endif
#ifndef INIT
#define INIT
#endif
#endif

/* location of substitution matrix (Dayhoff, PAM, BLOSUM, Gonnet) files */
#define BLASTMAT "/usr/ncbi/blast/matrix/"
/* location of database files */
#define BLASTDB "/usr/ncbi/blast/db/"
/* location of sequence filter programs */
#define BLASTFILTER "/usr/ncbi/blast/filter/"
/* location of codon frequency information files */
#define BLASTCDI "/usr/ncbi/blast/cdi/"

#if 0
#ifdef OS_UNIX
/* qsort based on the BSD UNIX version provides 2X faster sorting */
#define hsort qsort
#endif
#endif

#define AMINO_ACID_SEQTYPE	1
#define AA_SEQTYPE	AMINO_ACID_SEQTYPE
#define NUCLEIC_ACID_SEQTYPE	2
#define NT_SEQTYPE	NUCLEIC_ACID_SEQTYPE

/* Filename extensions used by the two types of databases (a.a. and nt.) */
#define AA_HEADER_EXT	".ahd"
#define AA_TABLE_EXT	".atb"
#define AA_SEARCHSEQ_EXT	".bsq"
#define NT_HEADER_EXT	".nhd"
#define NT_TABLE_EXT	".ntb"
#define NT_SEARCHSEQ_EXT	".csq"

#ifdef BLASTP
#define QUERY_SEQTYPE	AMINO_ACID_SEQTYPE
#define DATABASE_SEQTYPE	AMINO_ACID_SEQTYPE
#define DEFAULT_E_MAX	1000.
#endif
#ifdef TBLASTN
#define QUERY_SEQTYPE	AMINO_ACID_SEQTYPE
#define DATABASE_SEQTYPE	NUCLEIC_ACID_SEQTYPE
#define DEFAULT_E	5.
#define DEFAULT_E_MAX	1000.
#endif
#ifdef BLASTN
#define QUERY_SEQTYPE	NUCLEIC_ACID_SEQTYPE
#define DATABASE_SEQTYPE	NUCLEIC_ACID_SEQTYPE
#define DEFAULT_E_MAX	1000.
#define PROC_MAX	4
#endif
#ifdef BLASTX
#define QUERY_SEQTYPE	NUCLEIC_ACID_SEQTYPE
#define DATABASE_SEQTYPE	AMINO_ACID_SEQTYPE
#define DEFAULT_E	5.
#define DEFAULT_E_MAX	1000.
#endif
#ifdef BLAST3
#define QUERY_SEQTYPE	AMINO_ACID_SEQTYPE
#define DATABASE_SEQTYPE	AMINO_ACID_SEQTYPE
#define DEFAULT_E	1000.	/* Need many insignificant HSPs for 3-way stuff */
#endif

#ifndef PROC_MAX
#define PROC_MAX	THREADS_MAX
#endif

#if QUERY_SEQTYPE == NT_SEQTYPE && DATABASE_SEQTYPE == NT_SEQTYPE
#define WORDSIZE_MIN	8		/* min. length of critical word */
#define WORDSIZE_MAX	12		/* max. length of critical word */
#else
#define WORDSIZE_MIN	1		/* min. length of critical word */
#define WORDSIZE_MAX	12		/* max. length of critical word */
#endif

#ifndef QUERYLEN_MAX
#if QUERY_SEQTYPE == NT_SEQTYPE
#define QUERYLEN_MAX	(1000000)
#else
#define QUERYLEN_MAX	35000
#endif
#endif /* !QUERYLEN_MAX */

#ifdef DATABASE_SEQTYPE
#if DATABASE_SEQTYPE == AMINO_ACID_SEQTYPE
#define TABLE_EXT	AA_TABLE_EXT
#define HEADER_EXT	AA_HEADER_EXT
#endif
#if DATABASE_SEQTYPE == NUCLEIC_ACID_SEQTYPE
#define TABLE_EXT	NT_TABLE_EXT
#define HEADER_EXT	NT_HEADER_EXT
#endif
#endif


/* Define the common default parameters */
#ifndef DEFAULT_E
#define DEFAULT_E	10.		/* default expect for reporting HSPs */
#endif
#ifndef DEFAULT_S
#define DEFAULT_S	1		/* default cutoff for reporting HSPs */
#endif
#ifndef DEFAULT_V
#define DEFAULT_V	500
#endif
#ifndef DEFAULT_C
#define DEFAULT_C	1		/* default genetic code is Standard */
#endif
#ifndef REMEMBER_MAX
#define REMEMBER_MAX	10000 /* max. no. of database sequences to remember */
#endif
#ifndef DEFAULT_showhist
#define DEFAULT_showhist	0
#endif
#ifndef DEFAULT_showblast
#define DEFAULT_showblast	250
#endif
#ifndef BLASTDB
#define BLASTDB	"/usr/ncbi/blast/db/"	/* directory containing BLAST-able databases */
#endif
#ifndef BLASTMAT
/* directory of substitution matrix (Dayhoff, PAM, BLOSUM, Gonnet) files */
#define BLASTMAT "/usr/ncbi/blast/matrix/"
#endif
#ifndef GAPDECAYRATE_DEFAULT
#define GAPDECAYRATE_DEFAULT 0.5
#endif

#define NTICKS	50 /* Max. number of ticks to show in the search progress */

#define PLURAL(i)	((i) == 1 ? "" : "s")

#include "error.h"
#include "karlin.h"
#include "alphabet.h"

#define _DFA_PRIVATE_
#include <dfa.h>

enum score_type {
	SCORET_UNDEFINED = 0,
	SCORET_EXPECT = 1,
	SCORET_PVALUE = 2
	};

/* define REFINED_STATS for refined calculations of Expects, P-values,
and cutoff scores...
High-scoring alignments found with a short query sequence tend to involve
a large fraction (if not all) of the query sequence.  Given its score, the
expected length of a high-scoring segment pair significantly reduces the
number of degrees of freedom for producing such high scores with the short
query sequence.  The Expect and Poisson P-values should be reduced
appropriately.  This is done by subtracting the expected length of an
HSP (determined by its score) from the length of the query sequence
and from the length of the target database sequence, before calculating
the Expect and P-value.  In a similar manner, the cutoff score used
for a search is calculated from the user's choice (or the default value)
for the parameter E.
*/
#ifndef REFINED_STATS
#define REFINED_STATS
#endif

typedef long	Seqid_t;	/* sequence identifiers (must be signed) */

typedef long	Coord_t;	/* sequence coordinates (must be signed) */
#define COORD_MAX	INT4_MAX
#define COORD_MIN	INT4_MIN

typedef long	Diag_t;		/* 2-d matrix diagonal identifier */
#define DIAG_MAX	INT4_MAX
#define DIAG_MIN	INT4_MIN

#ifndef BLASTN
typedef int	DIAG_TYPE;
typedef int	*DIAG_PTR;
#else /* BLASTN */
/* blastn does things differently, because sequences may be very long. */
/* This structure retains the extents of matches for current sequence */
/* The approach used is inefficient if both the query and the longest
    database sequence are each quite long */
typedef struct diag_bnd {
		long	diag, pos;
	} DIAG_TYPE;
typedef int	DIAG_PTR;
#define DIAGCNT_MAX	(200*1024)
#endif

typedef unsigned char	PACK_TYPE;	/* Compressed nuc. sequence datatype */

/* seq_data is used by BLASTX, one copy per reading frame */
typedef struct {
		int		frame;
		char	*seq;
		DIAG_TYPE
				*_dlev,			/* extent of discovered matches... */
				*diag_level;	/*    on a given diagonal */
		DIAG_PTR
				diag_stack,		/* list of diagonals with matches */
				diag_top;		/* next diagonal with a match */
		long	len;			/* length of character string that is the seq */
		long	efflen;			/* effective length of sequence (for stats) */
		Score_t
				maxscore,	/* maximum achievable score */
				highs_exp,	/* expected high score */
				highs_obs,	/* observed high score */
				S0,			/* cutoff score for recording statistics */
				S,			/* cutoff score for this reading frame */
				X,			/* drop-off score for this reading frame */
				minusX,		/* negative X */
				T;
		int		stdframe;	/* standard frame number (+3, +2, +1, -1, -2, -3) */
		double	E;
		double	Lambda, K, H;
	} seq_data;

typedef struct hsp {
		struct hsp	PNTR next;
		VoidPtr	user; /* user-defined pointer */
		long	len, q_pos, s_pos;
		unsigned	n; /* n parameter to Poisson statistics */
		Score_t	score;
		double	P_val, expect;
		int		frame;		/* reading frame */
	} HSP, PNTR HSPPtr;

typedef struct hitlist {
		struct hitlist	*next;	/* hits against the next target sequence */
		VoidPtr	user;			/* user-defined pointer */
		Seqid_t	seq_nbr;		/* unique target sequence identifier */
#define HEADERLEN_MAX	200		/* max. stored length of a header line */
		int		header_len;		/* length of header (excl. null terminator) */
		char	*header;	/* string description of target sequence */
		unsigned	hspcnt;	/* no. of HSPs in the linked list */
		struct hsp	*hp;	/* list of HSPs for this target sequence */
		struct hsp	*signifhsp;/* single most significant HSP in the list*/
		Score_t	highscore, totalscore;
		float	lambda;
	} HitList, *HitListPtr;


typedef PoolBlkPtr	HitListPoolPtr;
typedef PoolBlkPtr	HSPPoolPtr;


#ifdef MPROC_AVAIL
	/* Multiprocessing context, one for each processor */
typedef struct mpcontext {
		int		id;	/* task id number */
		Score_t	cutoff;
		long
				numhits,
				numhits8,
				numfailed,
				overcnt,
				totHSP,
				nbr_seqs, nbr_seqs_max;
		Seqid_t	current; /* current db sequence being examined */
		DIAG_TYPE
				*_dlev,
				*diag_level;
		DIAG_PTR
				diag_stack,
				diag_top;
		char
				*s_seq0,
				*s_seq1,	/* used by TBLASTN for reading frame 2 */
				*s_seq2,	/* used by TBLASTN for reading frame 3 */
				*s_seq,
				*d_seq;
		PACK_TYPE	
				*c_seq,
				*c_seqp1,
				*c_lastp;
		Coord_t	
				s_len,
				d_len,
				c_len,
				c_last;
		int		c_pad, frame;
		Score_t	high_score, highs_obs;
		unsigned long	*score_hist; /* histogram of max. hit extension scores */
		HSPPoolPtr	poolp;
		HitListPoolPtr	hl_pool;
		HitList		curhl;
		HitListPtr	firsthl;
		seq_data	sd[6];
	} MPContext, *MPContextPtr;
#define MP	mp
#define MPC	mp,
#define MPPROTO	MPContextPtr
#define MPPROTOC	MPContextPtr,
#define MPDEF	MPContextPtr	mp;
#define MPPTR	mp->
#else /* !MPROC_AVAIL */
/* Dummy definitions when multiprocessing is not used */
#define MP
#define MPC
#define MPPROTO
#define MPPROTOC
#define MPDEF
#define MPPTR
#endif /* !MPROC_AVAIL */

#ifdef BLAST3
struct record {
		Score_t	score;
		Score_t	scor3;
		Seqid_t	seq2;
		Seqid_t	seq3;
		Coord_t	start1;
		Coord_t	dg2;
		Coord_t	dg3;
		Coord_t	length;
	};
#define RECORD_MAX	40000
#endif


#define NBPN	2					/* no. of bits per packed nucleotide */

#define TOP_STRAND	1
#define BOTTOM_STRAND	2

#define NT_MAGIC_BYTE	0xfc		/* Magic byte at end of compressed nt db */
#define NSENTINELS	2

void	banner PROTO((char *progname, char *version, char *rel_date, char *citation, char *notice));

int		get_weights PROTO((char *mname, int badletter, int *xltab,
							int *asize, int *minchar, int *maxchar,
							char (*achars)[ALPHASIZE_MAX+1],
							char (*order)[ALPHAVAL_MAX+1][ALPHASIZE_MAX],
							Score_t *lows, Score_t *highs,
							Score_t (*maxcost)[ALPHAVAL_MAX+1],
							Score_t (*matrx)[ALPHAVAL_MAX+1][ALPHAVAL_MAX+1]));

void	sort_HSPs PROTO((HitListPtr));
void	sort_hitlists_by_seqid PROTO((void));

void	get_seq_from_fasta PROTO((CharPtr buf, Coord_t start, unsigned long len, FILE *fp, off_t offset, int line_len));
void	get_seq_from_fasta_xl PROTO((CharPtr buf, Coord_t start, unsigned long len, FILE *fp, off_t offset, int line_len, int PNTR xltab));
void	get_seq_from_cseq PROTO((CharPtr buf, Coord_t start, unsigned long len, MFILE *mfp, unsigned char *c_seq0, off_t offset));
void	get_seq_from_cseq_xl PROTO((CharPtr buf, Coord_t start, unsigned long len, MFILE *mfp, unsigned char *c_seq0, off_t offset, CharPtr xltab));

void	get_headers PROTO((int dbtype, HitListPtr hlp));
int		print_headers PROTO((HitListPtr hlp, double maxpval, int hasframe, int linelen));
int		wrap PROTO((FILE *fp, char *pref, char *s, int slen, int linelen, int margin));

int		print_HSPs PROTO((HitListPtr hlp, double maxpval, int (*)()));

Score_t	fmaxscore PROTO((CharPtr seq, Coord_t seqlen, Score_t PNTR maxscore));
void	pcnt PROTO((HitListPtr));
void	consistn PROTO((HitListPtr));
void	cutoffs PROTO((Score_t *S, double *E, Score_t *highscore, double K, double Lambda, double H, unsigned long Neff, unsigned long Meff, double decayrate));
void	pvals PROTO((HitListPtr hlp, unsigned long qlen, unsigned long tlen, unsigned long dblen));
void	consist_pvals PROTO((HitListPtr hlp, unsigned long qlen, unsigned long tlen, unsigned long dblen));
void	evals PROTO((HitListPtr hlp, unsigned long qlen, unsigned long tlen, unsigned long dblen));
void	consist_evals PROTO((HitListPtr hlp, unsigned long qlen, unsigned long tlen, unsigned long dblen));
void	evals_x PROTO((HitListPtr hlp, unsigned long qlen, unsigned long dblen));
void	consist_evals_x PROTO((HitListPtr hlp, unsigned long qlen, unsigned long dblen));
void	pvals_x PROTO((HitListPtr hlp, unsigned long tlen, unsigned long dblen));
void	consist_pvals_x PROTO((HitListPtr hlp, unsigned long tlen, unsigned long dblen));
double	gapdecay PROTO((unsigned nsegs, double pvalue, double decayrate));
double	gapdecay_inverse PROTO((unsigned nsegs, double pvalue, double decayrate));
void	report_gapdecayrate PROTO((void));

int		seqfilter PROTO((CharPtr cmd, CharPtr outtab, int PNTR intab, CharPtr name, size_t namelen, UcharPtr inseq, size_t inlen, UcharPtr outseq, size_t PNTR outlen));
CharPtr _cdecl	pick_filter PROTO((CharPtr cmdstring, int restype));

int		hsp_print PROTO((FILE *fp, HSPPtr hp, CharPtr flagscore, UcharPtr q, UcharPtr s, Score_t mat[ALPHAVAL_MAX+1][ALPHAVAL_MAX+1], CharPtr q_btoa, CharPtr s_btoa, unsigned long q_len, unsigned long s_len, long (*fqpos0)(), long (*fqpos1)(), long (*fspos0)(), long (*fspos1)(), CharPtr framelabel, CharPtr (*str_frame)(int), double lambda));
long	nox_coord PROTO((unsigned long seqlen, int frame, unsigned long off, long adjust));
long	nt_coord PROTO((unsigned long seqlen, int frame, unsigned long off, long adjust));
long	apo_to_nt1 PROTO((unsigned long ntlen, int frame, unsigned long aaoff, long adjust));
long	apo_to_nt3 PROTO((unsigned long ntlen, int frame, unsigned long aaoff, long adjust));
CharPtr	str_frame PROTO((int frame));
CharPtr	str_strand PROTO((int frame));
void	printmid0 PROTO((FILE *fp, UcharPtr q, UcharPtr s, int len, Score_t mat[ALPHAVAL_MAX+1][ALPHAVAL_MAX+1], CharPtr q_btoa, CharPtr s_btoa, int width));
void	printmid1 PROTO((FILE *fp, UcharPtr q, UcharPtr s, int len, Score_t mat[ALPHAVAL_MAX+1][ALPHAVAL_MAX+1], CharPtr q_btoa, CharPtr s_btoa, int width));

DFAPtr	bld_table_aa PROTO((DFAPtr dp, int frame, char *qseq, size_t qlen,
						int W, Score_t T, int asize,
						Score_t (*matrix)[ALPHAVAL_MAX+1][ALPHAVAL_MAX+1],
						Score_t (*maxcost)[ALPHAVAL_MAX+1],
						char (*order)[ALPHAVAL_MAX+1][ALPHASIZE_MAX],
						unsigned long *numwords, unsigned long *numexact
						));
DFAPtr	bld_table_xa PROTO((DFAPtr dp, int frame, char *qseq, unsigned char gcode[64],
						int W, Score_t T, int asize,
						Score_t (*matrix)[ALPHAVAL_MAX+1][ALPHAVAL_MAX+1],
						Score_t (*maxcost)[ALPHAVAL_MAX+1],
						char (*order)[ALPHAVAL_MAX+1][ALPHASIZE_MAX],
						unsigned long *numwords, unsigned long *numexact
						));

void	end_table PROTO((DFAPtr));

void	sort_hitlists PROTO((HitListPtr PNTR first, int (*)()));

void	print_hist PROTO((Score_t score_hist_dim, unsigned long *score_hist,
							Score_t S, Score_t T,
							double lambda, double K, double H,
							Coord_t qlen, unsigned long dblen, unsigned long nbr_seqs));

size_t	hsp_count PROTO((HitListPtr hlp));
size_t	hitlist_count PROTO((HitListPtr hlp));
void		hitlist_truncate PROTO((HitListPtr PNTR hlpp, unsigned cnt));
void		hitlist_save PROTO((HitListPoolPtr pbp, HitListPtr PNTR head, HitListPtr hlp));
void		hitlist_prune_by_score PROTO((HitListPtr hlp, HSPPoolPtr poolp, Score_t cutoff));
void		hitlist_prune_by_cutoff PROTO((HitListPtr hlp, HSPPoolPtr poolp, enum score_type which, double cutoff));

/* Functions that detect (and perhaps reject) overlapping HSPs */
void	ckoverlap PROTO((MPPROTOC HSPPtr hp));
Boolean	ovlap_a PROTO((HSPPtr hp1, HSPPtr hp2));
Boolean	ovlap_a2 PROTO((HSPPtr hp1, HSPPtr hp2));
Boolean	ovlap_n PROTO((HSPPtr hp1, HSPPtr hp2));
Boolean	ovlap_n2 PROTO((HSPPtr hp1, HSPPtr hp2));
Boolean	ovlap_t PROTO((HSPPtr hp1, HSPPtr hp2));
Boolean	ovlap_t2 PROTO((HSPPtr hp1, HSPPtr hp2));
Boolean	ovlap_x PROTO((HSPPtr hp1, HSPPtr hp2));
Boolean	ovlap_x2 PROTO((HSPPtr hp1, HSPPtr hp2));
Boolean	ovlap_0 PROTO((HSPPtr hp1, HSPPtr hp2));
EXTERN Boolean	(*ovlap) PROTO((HSPPtr hp1, HSPPtr hp2))
#if defined(BLASTP) || defined(BLAST3)
	INITIAL(ovlap_a)
#endif
#if defined(BLASTN)
	INITIAL(ovlap_n)
#endif
#if defined(BLASTX)
	INITIAL(ovlap_x)
#endif
#if defined(TBLASTN)
	INITIAL(ovlap_t)
#endif
	;

EXTERN CharPtr	overlap_option;
EXTERN Boolean	gi_option;

EXTERN seq_data	sd[6];	/* (BLASTX only) */

void	hsp_save_default PROTO((MPPROTOC HSPPtr hp));	/* General alloc. and save of an HSP */
HSPPoolPtr	HSPPoolNew PROTO((void));
void	HSPPoolInit PROTO((HSPPoolPtr));
void	HSPPoolDestruct PROTO((HSPPoolPtr));
HSPPtr	HSPPoolGet PROTO((HSPPoolPtr));
void	HSPPoolReturn PROTO((HSPPoolPtr, HSPPtr));
void	HSPPoolReturnList PROTO((HSPPoolPtr, HSPPtr));

HitListPoolPtr	HitListPoolNew PROTO((void));

int		cmp_by_pval PROTO((HitListPtr *, HitListPtr *));
int		cmp_by_count PROTO((HitListPtr *, HitListPtr *));
int		cmp_by_highscore PROTO((HitListPtr *, HitListPtr *));
int		cmp_by_totalscore PROTO((HitListPtr *, HitListPtr *));
CharPtr	prt_pvalue PROTO((CharPtr buf, double pvalue, int width, int precision));

int		hsp_cmp_default PROTO((HSPPtr *, HSPPtr *));
int		cmp_hitlists_by_seqid PROTO((HitListPtr *, HitListPtr *));
int		ckseqtype PROTO((int seqtype, char *seq, Coord_t seqlen));

FILE	*ckopen PROTO((char *fname, char *mode, int die));
VoidPtr	ckalloc PROTO((size_t amt)),	/* storage allocator */
		ckrealloc PROTO((VoidPtr ptr,size_t amt)), /* realloc() */
		ckalloc0 PROTO((size_t amt));	/*   with zero initialization */

void	bfatal VPROTO((enum blast_error blastapp_errno, char *format, ...));
void	exit_code PROTO((enum blast_error));
void	warning VPROTO((char *format, ...));
void	ckwarnings PROTO((void));
void	SigTerm PROTO((int sig));
void	RunWild PROTO((char *s, unsigned long n, void (*func)()));
void	rlimit PROTO((void));

EXTERN char	*module;	/* Program name */

#define HSP_MAX	2000	/* Max. # of HSPs to record (usually per db seq) */

EXTERN CharPtr	filtercmd INITIAL("");

EXTERN double	Lambda, K,	/* Parameters for Karlin statistics */
				H;			/* Relative entropy of the p's and q's */

EXTERN Boolean L_set, K_set;

EXTERN char		dbfname[FILENAME_MAX];	/* Database file name */
EXTERN MFILE	*hfile;

/* Non-zero ==> show the hit-extension histogram */
EXTERN int	showhist INITIAL(DEFAULT_showhist);

/* Non-zero ==> show full pairwise alignments */
EXTERN long	showblast INITIAL(DEFAULT_showblast);

EXTERN Nlm_Boolean	echofilter_flag;

EXTERN unsigned long	Meff,		/* Effective length of the query sequence */
				Neff;		/* Effective length of the database */
EXTERN unsigned long *header_beg;	/* Start of header line for each db seq */
EXTERN unsigned long nbr_seqs;	/* No. of sequences in database */
				/* First, Last, and Current HitLists */
EXTERN HSPPoolPtr	poolp;
EXTERN PoolBlkPtr	hl_pool; /* HitList pool */
EXTERN HitList	curhl;
EXTERN HitListPtr	firsthl;
EXTERN unsigned long	*score_hist;	/* histogram of maximum hit extension scores */
EXTERN Score_t	score_hist_dim;	/* dimension of score_hist */
EXTERN Score_t	cutoff;		/* Cutoff score for reporting HSPs */
/* max. number of HSPs to report */
EXTERN long		V INITIAL(DEFAULT_V);
EXTERN Boolean	V_set;

EXTERN Coord_t	q_len;		/* Length of query sequence */
EXTERN Coord_t	q_lenmwm;	/* q_len - WORDSIZE_MIN */
EXTERN Seqid_t	current;	/* current database sequence being examined */
EXTERN int		frame;		/* current reading frame (or zero) */

/* Length of current database sequence */
EXTERN Coord_t	s_len INITIAL(-1);

EXTERN unsigned long	d_len;		/* Length of current translated database sequence */
EXTERN unsigned long	c_len;		/* Length of compressed database sequence */
EXTERN int		c_pad;		/* No. of pad nucleotides in compressed sequence */
EXTERN Coord_t	c_last;		/* Offset of last byte in compressed sequence */
EXTERN PACK_TYPE	*c_seq;	/* Pointer to compressed database sequence */
EXTERN PACK_TYPE	*c_seqp1;	/* c_seq - 1 */
EXTERN PACK_TYPE	*c_lastp;	/* Ptr to last byte in compressed sequence */
EXTERN long		overcnt;	/* No. of overlapping HSPs discarded */
EXTERN long		numhits;	/* No. of word hits against database */
EXTERN long		numhits8;	/* No. of 8-mer word hits (BLASTN only) */
EXTERN long		numfailed;	/* No. of failed hit extensions */
EXTERN Score_t	high_score;
EXTERN Score_t	highs_obs, highs_exp; /* Observed, Experimental high scores */
EXTERN unsigned long	maxHSP INITIAL(HSP_MAX);
EXTERN long		totHSP;		/* Total no. of HSPs found against the database */
EXTERN long		hasHSP;		/* No. of db sequences with at least one HSP */
EXTERN int		nprocs;		/* Count of the current number of processes */
EXTERN int		numprocs
#if defined(MPROC_AVAIL) && defined(THREADS_MAX)
	 INITIAL(PROC_MAX)
#endif
	;

#ifdef MPROC_AVAIL
EXTERN MPContext	mpc[THREADS_MAX];
#endif	/* MPROC_AVAIL */

EXTERN void	(*hsp_save) PROTO((MPPROTOC HSPPtr hp)) INITIAL(hsp_save_default);

EXTERN int (*hsp_cmp) PROTO((HSPPtr *h1, HSPPtr *h2)) INITIAL(hsp_cmp_default);

EXTERN int (*hitlist_cmp_func)() INITIAL(cmp_by_pval);

EXTERN void (*printmid) PROTO((FILE *fp, UcharPtr q, UcharPtr s, int len, Score_t mat[ALPHAVAL_MAX+1][ALPHAVAL_MAX+1], CharPtr q_btoa, CharPtr s_btoa, int width))
#ifdef BLASTN
	INITIAL(printmid1)
#else
	INITIAL(printmid0)
#endif
	;

EXTERN double	gapdecayrate INITIAL(GAPDECAYRATE_DEFAULT);

EXTERN long	Qlen, Qoffset, Slen, Soffset;
EXTERN long	dbrecmin INITIAL(-1);
EXTERN long	dbrecmax INITIAL(-1);
EXTERN long	dbreccnt;

#define DB_TYPE_PRO	0x78857a4f	/* Magic # for a protein sequence database */
#define DB_TYPE_NUC	0x788325f8	/* Magic # for a nt. sequence database */

#define AAFORMAT	3	/* Latest a.a. database format ID number */
#define NTFORMAT	6	/* Latest nt. database format ID number */

#if defined(__DATE__) && defined(__TIME__)
EXTERN CharPtr
		cc_date INITIAL(__DATE__),
		cc_time INITIAL(__TIME__);
#endif /* defined(__DATE__) && defined(__TIME__) */

#endif /* __BLAST_H__ */
