#include <ncbi.h>
#include <gishlib.h>
#include "blastapp.h"

#define LINELEN	80	/* no. of characters per line */
#define IDLEN_MAX	25	/* maximum length for a database identifier */

static int	print_header PROTO((HitListPtr, int, int, int));
static int	getmaxidlen PROTO((HitListPtr));

/* Print all headers where the smallest P_val in the list is below a cutoff */
int
print_headers(hlp, maxpval, hasframe, linelen)
	register HitListPtr	hlp;
	register double	maxpval;
	int	hasframe;
	int	linelen;
{
	register int	len;
	long	cnt, noprt;
	int	descriptlen;
	int	maxidlen;
	char	*cp;

	if (V == 0)
		return 0;

	descriptlen = linelen - 20;
	printf("\n%*s\n", linelen-3, "Smallest");
	printf("%*s\n", linelen-4, "Poisson");
	printf("%*s\n", linelen-1,
		(hasframe ? "Reading  High  Probability" : "High  Probability"));
	if (hasframe) {
		cp = "Frame Score  P(N)      N";
		descriptlen -= 3;
		len = descriptlen - 3;
	}
	else {
		cp = "Score  P(N)      N";
		len = descriptlen;
	}
	printf("%*s %s\n\n",
			-len,
			"Sequences producing High-scoring Segment Pairs:",
			cp);

	if (hlp == NULL) {
		printf("\n      *** NONE ***\n\n");
		return 0;
	}

	maxidlen = getmaxidlen(hlp);

	/* For each HitList in the linked list... */
	for (cnt = noprt = 0; hlp != NULL; hlp = hlp->next) {
		if (hlp->signifhsp->P_val <= maxpval) {
			if (V < 0 || cnt < V) {
				++cnt;
				if (print_header(hlp, hasframe, descriptlen, maxidlen) != 0)
					return -1;
			}
			else
				++noprt;
		}
	}

	noprt += hasHSP - cnt;
	if (V > 0 && noprt > 0) {
		printf("\n");
		warning("Descriptions of %s database sequences were not reported due to the limiting value of parameter V = %ls.",
				ld2s(noprt),
				ld2s(V));
	}
	return printf("\n");
}


static int
print_header(hlp, hasframe, descriptlen, maxidlen)
	HitListPtr	hlp;
	int	hasframe;
	int	descriptlen;
	int	maxidlen;
{
	register int	len;
	register char	*cp, *ocp;
	char	*header0;
	int		header_len;
	double	p;
	char	ch;

	if ((cp = header0 = hlp->header) == NULL)
		return 0;
	if ((len = header_len = hlp->header_len) == 0)
		return 0;

	if (maxidlen > 0) {
		descriptlen -= (maxidlen+1);
		/* Find first white space */
		ocp = cp;
		while ((ch = *ocp) != NULLB && !isspace(ch))
			++ocp;
		maxidlen -= (ocp - cp);
		/* Output the identifier */
		while (cp < ocp) {
			putc(*cp, stdout);
			++cp;
		}
		/* Pad with white space to align the left edge of the descriptions */
		while (maxidlen-- >= 0)
			putc(' ', stdout);
		/* Skip past any white space following the identifier */
		while ((ch = *cp) != NULLB && isspace(ch))
			++cp;
		len = header_len - (cp - header0);
	}
	if (len > descriptlen) {
		ocp = cp + (descriptlen-3);
		while (cp < ocp && (ch = *cp++) != NULLB)
			putc(ch, stdout);
		putc('.', stdout); putc('.', stdout); putc('.', stdout);
	}
	else
		printf("%*s", -descriptlen, cp);

	if (hasframe)
		printf(" %+d", hlp->hp->frame);
	printf(" %5ld", hlp->highscore);
	p = hlp->signifhsp->P_val;
	if (p <= 0.99) {
		if (p != 0.)
			printf("  %#-8.2lg", p);
		else
#if 0
			printf(" <%#-8.2lg", fct_dbl_min());
#else
			printf("  %#-8.2lg", 0.);
#endif
	}
	else if (p <= 0.999)
		printf("  %#-8.3lg", p);
	else if (p <= 0.9999)
		printf("  %#-8.4lg", p);
	else if (p <= 0.99999)
		printf("  %#-8.5lg", p);
	else if (p <  0.9999995)
		printf("  %#-8.6lg", p);
	else
		printf("  %#-8.7lg", 1.0);
	printf(" %2d", hlp->signifhsp->n);
	putc('\n', stdout);
	return ferror(stdout);
}


/* Determine the length of the longest database sequence identifier */
static int
getmaxidlen(hlp)
	register HitListPtr	hlp;
{
	register char	*cp, ch;
	register int	len, maxidlen;
	char	*header0;

	maxidlen = 0;
	for (; hlp != NULL; hlp = hlp->next) {
		if ((cp = hlp->header) == NULL)
			continue;
		header0 = hlp->header;
		while ((ch = *cp) != NULLB && !isspace(ch))
			++cp;
		len = cp - header0;
		if (len > maxidlen) {
			if ((maxidlen = len) > IDLEN_MAX)
				break;
		}
	}

	if (maxidlen > IDLEN_MAX)
		maxidlen = 0; /* too long... looks like a non-standard database */

	return maxidlen;
}
