#include <ncbi.h>
#include "blastapp.h"

/* -----------------   read file of substitution weights   ----------------*/

/*  get_weights() reads a file giving weights for amino acid substitutions.
The proper form of the file is illustrated by the following "PAM120" matrix.
The first row gives amino acid abbreviations, which must be single capital
letters.  Weights should be small integers (e.g., between -128 and 127).

   A  R  N  B  D  C  Q  Z  E  G  H  I  L  K  M  F  P  S  T  W  Y  V  X  *
A  3 -3  0  0  0 -3 -1 -1  0  1 -3 -1 -3 -2 -2 -4  1  1  1 -7 -4  0 -1 -8
R -3  6 -1 -2 -3 -4  1 -1 -3 -4  1 -2 -4  2 -1 -4 -1 -1 -2  1 -6 -3 -1 -8
N  0 -1  4  3  2 -5  0  0  1  0  2 -2 -4  1 -3 -4 -2  1  0 -5 -2 -3 -1 -8
B  0 -2  3  4  3 -6  0  1  2  0  1 -3 -5  0 -4 -6 -2  0 -1 -7 -4 -3 -1 -8
D  0 -3  2  3  5 -7  1  2  3  0  0 -3 -5 -1 -4 -7 -2  0 -1 -8 -5 -3 -1 -8
C -3 -4 -5 -6 -7  9 -7 -7 -7 -5 -4 -3 -7 -7 -6 -6 -3 -1 -3 -8 -1 -2 -1 -8
Q -1  1  0  0  1 -7  6  4  2 -3  3 -3 -2  0 -1 -6  0 -2 -2 -6 -5 -3 -1 -8
Z -1 -1  0  1  2 -7  4  5  3 -2  1 -3 -3 -1 -3 -6 -1 -2 -2 -7 -5 -3 -1 -8
E  0 -3  1  2  3 -7  2  3  5 -1 -1 -3 -4 -1 -4 -6 -1 -1 -2 -8 -4 -3 -1 -8
G  1 -4  0  0  0 -5 -3 -2 -1  5 -4 -4 -5 -3 -4 -5 -2  1 -1 -8 -6 -2 -1 -8
H -3  1  2  1  0 -4  3  1 -1 -4  7 -4 -3 -2 -4 -2 -1 -2 -3 -5 -1 -3 -1 -8
I -1 -2 -2 -3 -3 -3 -3 -3 -3 -4 -4  6  1 -2  1  0 -3 -2  0 -7 -2  3 -1 -8
L -3 -4 -4 -5 -5 -7 -2 -3 -4 -5 -3  1  5 -4  3  0 -3 -4 -3 -5 -3  1 -1 -8
K -2  2  1  0 -1 -7  0 -1 -1 -3 -2 -2 -4  5  0 -6 -2 -1 -1 -5 -6 -4 -1 -8
M -2 -1 -3 -4 -4 -6 -1 -3 -4 -4 -4  1  3  0  8 -1 -3 -2 -1 -7 -4  1 -1 -8
F -4 -4 -4 -6 -7 -6 -6 -6 -6 -5 -2  0  0 -6 -1  8 -5 -3 -4 -1  4 -3 -1 -8
P  1 -1 -2 -2 -2 -3  0 -1 -1 -2 -1 -3 -3 -2 -3 -5  6  1 -1 -7 -6 -2 -1 -8
S  1 -1  1  0  0 -1 -2 -2 -1  1 -2 -2 -4 -1 -2 -3  1  3  2 -2 -3 -2 -1 -8
T  1 -2  0 -1 -1 -3 -2 -2 -2 -1 -3  0 -3 -1 -1 -4 -1  2  4 -6 -3  0 -1 -8
W -7  1 -5 -7 -8 -8 -6 -7 -8 -8 -5 -7 -5 -5 -7 -1 -7 -2 -6 12 -1 -8 -1 -8
Y -4 -6 -2 -4 -5 -1 -5 -5 -4 -6 -1 -2 -3 -6 -4  4 -6 -3 -3 -1  8 -3 -1 -8
V  0 -3 -3 -3 -3 -2 -3 -3 -3 -2 -3  3  1 -4  1 -3 -2 -2  0 -8 -3  5 -1 -8
X -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -8
* -8 -8 -8 -8 -8 -8 -8 -8 -8 -8 -8 -8 -8 -8 -8 -8 -8 -8 -8 -8 -8 -8 -8  1

*/


int
get_weights(mname, lettermax, xltab, asize, minchar, maxchar,
			achars, order, lows, highs, maxcost, matrx)
	char	*mname;	/* Name of file containing the substitution matrix */
	int		lettermax;	/* Max binary letter value used to screen input file */
	int		*xltab;	/* ASCII to binary translation table */
	int		*asize;	/* No. of letters in the alphabet */
	int		*minchar, *maxchar;	/* Min. and max. letter values */
	char
		/* array of amino acid names, plus null terminator */
		(*achars)[ALPHASIZE_MAX+1],
		/* order[i][j] = j-th best char to sub. for i */
		(*order)[ALPHAVAL_MAX+1][ALPHASIZE_MAX];

	Score_t
		*lows,			/* lowest substitution value */
		*highs,			/* highest substitution value */
		/* max cost of substituting for a character */
		(*maxcost)[ALPHAVAL_MAX+1],
		/* matrx[i][j] = cost of replacing i by j */
		(*matrx)[ALPHAVAL_MAX+1][ALPHAVAL_MAX+1];
{
	char buf[FILENAME_MAX], *cp;
	FILE	*fp;
	int		i, j, lineno = 1, chcnt, chno, ch, thechar;
	Score_t		score;
	double	xscore;
	Score_t	*m;
	char	*o, *p, *q, t;
	char	once=0;

Retry:
	if ((fp = ckopen(mname, "r", 0)) == NULL) {

		if (++once > 1)
			bfatal(ERR_FOPEN, "Could not find or open a scoring matrix file named:  %s", mname);
		cp = getenv("BLASTMAT");
		sprintf(buf, "%s%s%s%s%s",
				(cp == NULL ? BLASTMAT : cp),
				DIRDELIMSTR,
				"aa",
				DIRDELIMSTR,
				mname);
		mname = buf;
		goto Retry;
	}

	/* Ignore any initial comment line(s) */
	while ((i = fgetc(fp)) == '#') {
		char	buf[1024];
		++lineno;
		if (fgets(buf, sizeof buf, fp) == NULL)
			break;
	}
	ungetc(i, fp);

	/* read residue names */
	*minchar = INT_MAX;
	*maxchar = INT_MIN;
	p = (*achars);
	while ((i = fgetc(fp)) != '\n' && i != 0) {
		if ((j = xltab[i]) < lettermax) {
			if (p - (*achars) >= DIM(*achars)-1)
				bfatal(ERR_SUBFILE, "Too many amino acid names in scoring matrix file %s.", mname);
			if (j > *maxchar)
				*maxchar = j;
			if (j < *minchar)
				*minchar = j;
			*p++ = j;
		}
		else {
			if (isspace(i))
				continue;
			if (isprint(i))
				bfatal(ERR_SUBFILE, "Error reading scoring matrix file %s:  Bad a.a. letter: ``%c.''", mname, i);
			else
				bfatal(ERR_SUBFILE, "Error reading scoring matrix file %s:  Bad a.a. letter: #%d.", mname, i);
		}
	}

	*asize = p - (*achars);
	if (*asize >= DIM(*matrx))
		bfatal(ERR_SUBFILE, "too many amino acid names in scoring matrix file %s.", mname);

	/* For illegal characters, set weight to lowest possible value. */
	for (i=0; i<DIM(*maxcost); ++i)
		(*maxcost)[i] = SCORE_MIN/2;
	for (i=0; i<DIM(*matrx); ++i)
		for (j=0; j<DIM((*matrx)[0]); ++j)
			(*matrx)[i][j] = SCORE_MIN/WORDSIZE_MAX;

	/* read substitution weights for each residue */
	*lows = SCORE_MAX;
	*highs = SCORE_MIN;
	for (chcnt = 0; chcnt < *asize; ++lineno) {
		ch = fgetc(fp);
		if (ch == '\n')
			continue;
		if (ch == EOF || isspace(ch))
			bfatal(ERR_SUBFILE, "Missing letter in scoring matrix file ``%s'', line %d.", mname, lineno);
		++chcnt;
		thechar = xltab[ch];
		for (chno = 0; chno < *asize; ++chno)
			if (thechar == (*achars)[chno])
				break;
		if (chno >= *asize)
			bfatal(ERR_SUBFILE, "Unexpected letter in scoring matrix file ``%s'', line %d.", mname, lineno);
		m = (*matrx)[thechar];
		for (j = 0; j < *asize; ++j) {
			if (fscanf(fp, "%lg", &xscore) != 1)
				bfatal(ERR_SUBFILE, "Numeric conversion error reading scoring matrix file ``%s'', on line %d.",
							mname, lineno);
			score = fct_nint(xscore);
			m[(*achars)[j]] = score;
			if (*lows > score)
				*lows = score;
			if (*highs < score)
				*highs = score;
		}
		while ((ch = fgetc(fp)) != '\n' && ch != EOF)
			;
		o = (*order)[thechar];
		for (j = 0; j < *asize; ++j)
			o[j] = (*achars)[j];
		/* bubble-sort names by cost of substitution for chcnt-th residue */
		for (q = o + (*asize-1); q > o; --q)
			for (p = o; p < q; ++p)
				if (m[*p] < m[*q]) {
					t = *p;
					*p = *q;
					*q = t;
				}
		if ((*maxcost)[thechar] > SCORE_MIN/2)
			bfatal(ERR_SUBFILE, "Duplicate letter specification in matrix file ``%s'', line %d.", mname, lineno);
		(*maxcost)[thechar] = m[*o];
	}
	(void) fclose(fp);

	return 0;
}
