/* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================*/

#include <ncbi.h>
#include <dfa.h>

/*************************************************************************
 *	dfa_pscanf(dp, gettoken, stream, report)
 *
 *	Use the specified DFA to scan a stream of tokens and report matches.
 *************************************************************************/
DFA_Error _cdecl
dfa_pscanf(dp, gettoken, stream, report)
	DFAPtr	dp;
	/* Call-back function to get each token */
	int	(_cdecl *gettoken) PROTO((Nlm_VoidPtr _stream));
	Nlm_VoidPtr	stream;
	/* Call-back function to report matches */
	int	(_cdecl *report) PROTO((size_t off, DFA_PatID patID));
{
	DFA_StatePtr	S;
	DFA_AcceptPtr	ap;
	DFA_PatlistPtr	plp;
	size_t	off = 0;
	int	ch;

	if (dp == NULL || dp->magic != DFA_MAGIC)
		return dfaerrno = dfaErrBadPtr;
	if (dfa_opstate(dp) != dfaOpstateClosed)
		return dfaerrno = dfaErrOpstate;
	if (dp->xlen != 0)
		return dfaerrno = dfaErrIncompat;

	if (gettoken == NULL || report == NULL)
		return dfaerrno = dfaErrNullParm;

	S = dp->state0;

	for (;; ++off) {
		while (!DFA_ISACCEPTING(S = S->next[ch = (*gettoken)(stream)])) {
			++off;
			if (ch == EOF)
				return dfaErrNone;
		}
		if ((ap = (DFA_AcceptPtr)DFA_UNMUNGED(S)) == NULL)
			return dfaErrNone;
		S = ap->retState;
		plp = &ap->pl;
		do {
			if ((*report)(off, plp->patID) != 0)
				return dfaerrno = dfaErrReport;
		} while ((plp = plp->chain) != NULL);
	}
	/*NOTREACHED*/
}
