#include <ncbi.h>
#include <gishlib.h>
#include <signal.h>
#ifdef MPROC_AVAIL
#include <sys/resource.h>
#endif

static int		proc_max = THREADS_MAX,
				proc_reserve = PROC_RESERVE,
				nchunks = NCHUNKS;

static int		m_next_cnt;

typedef struct {
			void		(_cdecl *func)();
			VoidPtr		userp;
			TaskBlkPtr	tp;
			int			PNTR nprocs;
			void		(*sigterm)();
			int			id;
		} WildBlk, PNTR WildBlkPtr;

#ifdef MPROC_AVAIL
#ifdef SOLARIS_THREADS_AVAIL
static mutex_t	_next_mutex;

static int
m_next()
{
	int		retcode;

	mutex_lock(&_next_mutex);
	retcode = m_next_cnt++;
	mutex_unlock(&_next_mutex);
	return retcode;
}
#endif /* SOLARIS_THREADS_AVAIL */

#ifdef DCE_THREADS_AVAIL
static pthread_mutex_t	_next_mutex;

static int
m_next()
{
	int		retcode;

	pthread_mutex_lock(&_next_mutex);
	retcode = m_next_cnt++;
	pthread_mutex_unlock(&_next_mutex);
	return retcode;
}
#endif /* DCE_THREADS_AVAIL */

#endif /* MPROC_AVAIL */

static void
_mticknext(tp, n)
	TaskBlkPtr	tp;
	unsigned long	n;
{
	if (n >= *tp->gtick_cnt && *tp->gtick_out < tp->nticks) {
		M_LOCK;
		while (n > *tp->gtick_cnt && *tp->gtick_out < tp->nticks) {
			*(tp->gtick_cnt) += tp->tick_incr;
			++*(tp->gtick_out);
			if (tp->tickproc != NULL)
				(*tp->tickproc)(tp->userp);
		}
		M_UNLOCK;
	}
}

static long
_mtasknext(tp)
	TaskBlkPtr	tp;
{
	if (++tp->task_cur >= tp->task_max) {
		tp->task_cur = m_next() * tp->chunksize;
		if (tp->task_cur >= tp->ntasks)
			return tp->task_cur = LONG_MIN;
		tp->task_max = tp->task_cur + tp->chunksize;
		tp->task_max = MIN(tp->task_max, tp->ntasks);
	}
	if (tp->tickproc == NULL)
		return tp->task_cur;
	_mticknext(tp, tp->task_cur);
	return tp->task_cur;
}


static void
wild_init(wbp)
	WildBlkPtr	wbp;
{
	void	(_cdecl *func)();
	Nlm_VoidPtr	userp;
	TaskBlkPtr	tp, tp_orig;
	int		PNTR nprocs;
#ifdef MPROC_AVAIL
#ifdef SGI_MPROC_AVAIL
	struct rlimit	rl;
	unsigned long	cputime = RLIM_INFINITY;
#endif
#endif
	int		id = 0;

#ifdef DCE_THREADS_AVAIL
	/* Synchronize with master thread */
	pthread_mutex_lock(&_gish_mutex);
	pthread_mutex_unlock(&_gish_mutex);
#endif

	func = wbp->func;
	userp = wbp->userp;
	tp_orig = wbp->tp;
	nprocs = wbp->nprocs;

#ifdef MPROC_AVAIL
#ifdef SGI_MPROC_AVAIL
	M_LOCK;
	tp_orig->nprocs++;
	tp_orig->proc_max = tp_orig->nprocs;
	++*nprocs;
	M_UNLOCK;
	id = m_get_myid();
	if (id == 0)
		signal(SIGTERM, wbp->sigterm);
	m_sync();
#endif /* SGI_MPROC_AVAIL */
#if defined(SOLARIS_THREADS_AVAIL) || defined(DCE_THREADS_AVAIL)
	id = wbp->id;
	signal(SIGTERM, wbp->sigterm);
#endif

	tp = TaskLink(tp_orig, id);
	tp->tasknext = _mtasknext;
#else /* !MPROC_AVAIL */

	tp = tp_orig;

#endif /* !MPROC_AVAIL */

	TaskInit(tp, nchunks);

#ifdef MPROC_AVAIL
#ifdef SGI_MPROC_AVAIL
	if (getrlimit(RLIMIT_CPU, &rl) != -1 && rl.rlim_cur != RLIM_INFINITY) {
		cputime = rl.rlim_cur;
		rl.rlim_cur = cputime / tp->proc_max + 1;
		setrlimit(RLIMIT_CPU, &rl);
	}
#endif
#endif

	(*func)(userp, tp);

#ifdef MPROC_AVAIL
#ifdef SGI_MPROC_AVAIL
	if (cputime != RLIM_INFINITY) {
		rl.rlim_cur = cputime;
		setrlimit(RLIMIT_CPU, &rl);
	}
#endif
#endif

	M_LOCK;
	--*nprocs;
	tp_orig->nprocs--;
	TaskUnlink(tp);
	M_UNLOCK;
}

int
mrun_wild(n, func, userp, tickproc, nticks, numprocs, nprocs, sigterm)
	unsigned long	n; /* Number of discrete subtasks */
	void	(_cdecl *func)();
	Nlm_VoidPtr	userp;
	void	(_cdecl *tickproc)();
	unsigned long	nticks;
	int		numprocs;
	int		PNTR nprocs;
	void	(_cdecl *sigterm)();
{
	TaskBlkPtr	tp;

#ifdef MPROC_AVAIL
#ifdef SGI_MPROC_AVAIL
	WildBlk		wb;
	int		maxprocs, maxpprocs;

	if (n == 0)
		return 0;

	maxprocs = prctl(PR_MAXPROCS); /* max. user processes */
	maxpprocs = prctl(PR_MAXPPROCS); /* max. parallel processes */
	numprocs = MAX(numprocs, 1);
	numprocs = MIN(numprocs, proc_max);
	numprocs = MIN(numprocs, maxprocs);
	numprocs = MIN(numprocs, maxpprocs - proc_reserve);
	numprocs = MAX(1, numprocs);
	m_set_procs(numprocs);

	(void) prctl(PR_SETEXITSIG, SIGTERM); /* signal SIGTERM if a child exits */

	tp = TaskNew(n, userp, tickproc, nticks);
	if (tp != NULL)
		tp->ticknext = _mticknext;
	wb.func = func;
	wb.userp = userp;
	wb.tp = tp;
	wb.nprocs = nprocs;
	wb.sigterm = sigterm;
	*nprocs = 0;
	if (m_fork(wild_init, &wb) != 0)
		return 0;
	m_kill_procs();
	signal(SIGTERM, SIG_DFL);
	numprocs = tp->proc_max;
	TaskDestruct(tp);
	return numprocs;
#endif /* SGI_MPROC_AVAIL */
#ifdef SOLARIS_THREADS_AVAIL
	thread_t	thr_id[THREADS_MAX];
	WildBlk	wb[THREADS_MAX];
	int		i;
	int		maxprocs;
	static int	once;

	if (n == 0)
		return 0;

	numprocs = MAX(numprocs, 1);
	numprocs = MIN(numprocs, proc_max);
	maxprocs = sysconf(_SC_NPROCESSORS_ONLN);
	maxprocs = MIN(numprocs, maxprocs - proc_reserve);
	maxprocs = MIN(maxprocs, DIM(wb));
	Nlm_MemSet((VoidPtr)wb, 0, maxprocs * sizeof(wb[0]));
	if (!once) {
		once = 1;
		mutex_init(&_gish_mutex, USYNC_THREAD, NULL);
		mutex_init(&_next_mutex, USYNC_THREAD, NULL);
	}

	tp = TaskNew(n, userp, tickproc, nticks);
	if (tp != NULL)
		tp->ticknext = _mticknext;
	fflush(stdout);

	m_next_cnt = 0;
	*nprocs = 0;
	for (numprocs = 0, i = 0; i < maxprocs; ++i) {
		thr_id[numprocs] = 0;
		if (thr_create(NULL, 0, (void *(*)(void *))wild_init, &wb[numprocs],
				THR_SUSPENDED, &thr_id[numprocs]) == 0)
			++numprocs;
	}
	tp->nprocs = numprocs;
	tp->proc_max = numprocs;
	thr_setconcurrency(numprocs);
	for (i = 0; i < numprocs; ++i) {
		wb[i].tp = tp;
		wb[i].func = func;
		wb[i].userp = userp;
		wb[i].tp = tp;
		wb[i].sigterm = sigterm;
		wb[i].nprocs = nprocs;
		wb[i].id = i;
		thr_continue(thr_id[i]);
	}
	for (i = 0; i < numprocs; ++i) {
		thread_t	th;
		thr_join((thread_t)0, &th, NULL);
	}
	signal(SIGTERM, SIG_DFL);
	TaskDestruct(tp);
	return numprocs;
#endif /* SOLARIS_THREADS_AVAIL */
#ifdef DCE_THREADS_AVAIL
	pthread_t	thr_id[THREADS_MAX];
	pthread_attr_t	thratt;
	pthread_addr_t	status;
	WildBlk	wb[THREADS_MAX];
	int		i;
	int		maxprocs, *start;
	static int	once;

	if (n == 0)
		return 0;

	numprocs = MAX(numprocs, 1);
	numprocs = MIN(numprocs, proc_max);
#ifdef _SC_NPROCESSORS_ONLN
	maxprocs = sysconf(_SC_NPROCESSORS_ONLN);
#else
#ifdef GSI_CPUS_IN_BOX
	start = NULL;
	getsysinfo(GSI_CPUS_IN_BOX, &maxprocs, sizeof(maxprocs), start, NULL);
	maxprocs = MAX(maxprocs, 1);
#else
	maxprocs = 1;
#endif
#endif
	maxprocs = MIN(numprocs, maxprocs - proc_reserve);
	maxprocs = MIN(maxprocs, DIM(wb));
	Nlm_MemSet((VoidPtr)wb, 0, maxprocs * sizeof(wb[0]));
	if (!once) {
		once = 1;
		pthread_mutex_init(&_gish_mutex, pthread_mutexattr_default);
		pthread_mutex_init(&_next_mutex, pthread_mutexattr_default);
	}

	tp = TaskNew(n, userp, tickproc, nticks);
	if (tp != NULL)
		tp->ticknext = _mticknext;
	fflush(stdout);

	m_next_cnt = 0;
	*nprocs = 0;
	pthread_attr_create(&thratt);
	pthread_attr_setstacksize(&thratt, 1*KBYTE*KBYTE);
	pthread_attr_setsched(&thratt,  SCHED_RR);
	pthread_setscheduler(pthread_self(), SCHED_RR, PRI_RR_MAX);
	pthread_mutex_lock(&_gish_mutex);
	for (numprocs = 0, i = 0; i < maxprocs; ++i) {
		if (pthread_create(&thr_id[numprocs], thratt,
				(void *(*)(void *))wild_init, &wb[numprocs]) == 0)
			++numprocs;
	}
	tp->nprocs = numprocs;
	tp->proc_max = numprocs;
	for (i = 0; i < numprocs; ++i) {
		wb[i].tp = tp;
		wb[i].func = func;
		wb[i].userp = userp;
		wb[i].tp = tp;
		wb[i].sigterm = sigterm;
		wb[i].nprocs = nprocs;
		wb[i].id = i;
	}
	pthread_mutex_unlock(&_gish_mutex); /* all threads synchronized */
	for (i = 0; i < numprocs; ++i) {
		pthread_join(thr_id[i], &status);
	}
	signal(SIGTERM, SIG_DFL);
	TaskDestruct(tp);
	return numprocs;
#endif

#else /* !MPROC_AVAIL */

	if (n == 0)
		return 0;

	tp = TaskNew(n, userp, tickproc, nticks);
	signal(SIGTERM, sigterm);
	*nprocs = 1;
	(*func)(userp, tp);
	*nprocs = 0;
	TaskDestruct(tp);
	return 1;

#endif /* !MPROC_AVAIL */
}
